/*
    Copyright (C) 2002 Kjetil S. Matheussen / Notam.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include <signal.h>
#include <stdio.h>

#include <stdlib.h>
#include <unistd.h>

#include <vstserver.h>

#include "clienthandler.h"


void *VSTSERVER_clientthread(void *arg){
  struct VSTS_serverRequest *sr=(struct VSTS_serverRequest *)arg;
  struct VSTClient *vstclient=CH_new(sr);

  if(vstclient!=NULL){
    int rec;
    char name[500];

    sprintf(name,"%s",vstclient->name);

    printf("VSTSERVER_clientthread: %s started.\n",name);
    if(C_receive(vstclient->newdelete_input,&rec,sizeof(int))==0){
      fprintf(stderr,"VSTSERVER/VSTSERVER_clientthread: Seems like client has died. Exiting.\n");
    }
    CH_delete(vstclient);
    printf("VSTSERVER_clientthread: %s stopped.\n",name);
  }

  /* Is it safe to free here? pthread_t for the current thread is placed in sr. */
  free(sr);

  return NULL;
}


static struct Communicate *input;

void finish(int sig){
  char temp[500];
  C_delete(input);  

  sprintf(temp,"rm -f /tmp/vstserver_socket_*");

  system(temp);

  exit(0);
}

int main(int argc,char **argv){


  input=C_newInput(0);

  if(input==NULL){
    fprintf(stderr,"\n\nVSTSERVER/main: Could not create main socket. Server is allready running\n");
    fprintf(stderr,"or there are left vstserver_socket* files in /tmp/. Just delete them manually.\n");
    fprintf(stderr,"Or, there is something seriously wrong. (hope not)\n\n");
    return 0;
  }

  signal(SIGINT,finish);

  printf("\n\nVSTSERVER/main: Vstserver started. Waiting for requests.\n");

  for(;;){
    struct VSTS_serverRequest *sr=malloc(sizeof(struct VSTS_serverRequest));

    for(;;){
      int size;
      while(C_acceptIncomingConnection(input)==false);

      if(
	 (size=C_receive(input,sr,sizeof(struct VSTS_serverRequest)))
	 != sizeof(struct VSTS_serverRequest)
	 )
	{
	  if(size!=-1){
	    fprintf(stderr,"VSTSERVER/main: Wrong type of request to vstserver. Size: %d\n",size);
	  }else{
	    fprintf(stderr,"VSTSERVER/main: Request failed.\n");
	  }
	}else{
	  break;
	}
    }

    switch(sr->reqtype){
    case VSTP_new:
      if(pthread_create(&sr->client_thread,NULL,VSTSERVER_clientthread,sr)!=0){
	fprintf(stderr,"VSTSERVER/main: Could not make new thread.\n");
	free(sr);
      }
      break;

    default:
      fprintf(stderr,"VSTSERVER/main: Illegal request.\n");
      break;
    }
  }




  return 0;

}

